using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX.Direct3D;

namespace blue_dish_te
{
	internal class Teleport : MapObject
	{
		private class intXY
		{
			public int x;
			public int y;

			public intXY(int x, int y)
			{
				this.x = x;
				this.y = y;
			}
		}

		public const float crossFadeTime = 0.5f;
		public float origFogDensity = 0;

		private bool fadeOut = false;
		private bool doCrossfade = false;
		private float elapsedTime = 0F;

		private Texture tt, tb;
		private VertexBuffer vb, vt;
		private ArrayList teleports = new ArrayList();

		//konstruktor objektu
		public Teleport(Device dev, DatLoader dat, char type) : base(dev, dat, type)
		{
			device = dev;
			origFogDensity = device.RenderState.FogDensity;

			//inicializace textury
			try
			{
				if ((tb = TextureLoader.FromStream(device, dat.GetItem("textury/teleportbottom.jpg"))) == null)
					throw new Exception("Nelze nacist texturu");
				if ((tt = TextureLoader.FromStream(device, dat.GetItem("textury/teleporttop.jpg"))) == null)
					throw new Exception("Nelze nacist texturu");
			}
			catch(Exception e)
			{
				MessageBox.Show("Nepovedlo se nacist texturu pro teleport.");
				Console.WriteLine(e);
				return;
			}

			CustomVertex.PositionNormalTextured[] v;

			//nacteni bodu
			vb = new VertexBuffer(typeof(CustomVertex.PositionNormalTextured), 5, device, Usage.WriteOnly, CustomVertex.PositionNormalTextured.Format, Pool.Managed);
			v = (CustomVertex.PositionNormalTextured[]) vb.Lock(0, 0);

			//naplneni VB souradnicemi vrcholu             x  y  z nx ny nz  u  v
			v[0] = new CustomVertex.PositionNormalTextured(2, 0, 0, 0, 1, 0, 1, 1);
			v[1] = new CustomVertex.PositionNormalTextured(0, 0, 0, 0, 1, 0, 0, 1);
			v[2] = new CustomVertex.PositionNormalTextured(2, 0, 2, 0, 1, 0, 1, 0);
			v[3] = new CustomVertex.PositionNormalTextured(0, 0, 0, 0, 1, 0, 0, 1);
			v[4] = new CustomVertex.PositionNormalTextured(0, 0, 2, 0, 1, 0, 0, 0);

			vb.Unlock(); //odemknuti VB 

			vt = new VertexBuffer(typeof(CustomVertex.PositionNormalTextured), 5, device, Usage.WriteOnly, CustomVertex.PositionNormalTextured.Format, Pool.Managed);
			v = (CustomVertex.PositionNormalTextured[]) vt.Lock(0, 0);

			v[0] = new CustomVertex.PositionNormalTextured(2, 3, 2, 0, -1, 0, 1, 1);
			v[1] = new CustomVertex.PositionNormalTextured(0, 3, 2, 0, -1, 0, 0, 1);
			v[2] = new CustomVertex.PositionNormalTextured(2, 3, 0, 0, -1, 0, 1, 0);
			v[3] = new CustomVertex.PositionNormalTextured(0, 3, 2, 0, -1, 0, 0, 1);
			v[4] = new CustomVertex.PositionNormalTextured(0, 3, 0, 0, -1, 0, 0, 0);
			vt.Unlock(); //odemknuti VB 
		}

		/// <summary>
		/// Vykresleni VertexBufferu jako trojuhelnikoveho stripu
		/// </summary>
		public override void Render()
		{
			// pokud je nutne kreslit prechod, tak ho nakreslime
			if (doCrossfade)
			{
				// provedeme vypocty
				int colorPart = 0;
				if (fadeOut)
					colorPart = (int) ((elapsedTime / (crossFadeTime / 2)) * 100);
				else
					colorPart = (int) (((crossFadeTime - elapsedTime) / (crossFadeTime / 2)) * 100);

				device.RenderState.FogDensity = origFogDensity + ((1 - origFogDensity) / 100) * colorPart;
				device.RenderState.FogColor = Color.FromArgb(colorPart, colorPart, colorPart);
				elapsedTime += renderTime;
				if (elapsedTime > crossFadeTime / 2 && fadeOut)
				{
					// pokud je u prostred prechodu skocime s hracem
					PlayerJump();
					fadeOut = false;
				}
				if (elapsedTime > crossFadeTime)
				{
					doCrossfade = false;
					device.RenderState.FogDensity = origFogDensity;
					device.RenderState.FogColor = Color.Black;
				}
			}

			// kresleni teleportu
			device.VertexFormat = CustomVertex.PositionNormalTextured.Format; //format dat

			if (!noTexture) device.SetTexture(0, tb);
			device.SetStreamSource(0, vb, 0); //nastaveni zdroje dat
			device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 3); //vykresleni VB jako TriangleStrip

			if (!noTexture) device.SetTexture(0, tt);
			device.SetStreamSource(0, vt, 0); //nastaveni zdroje dat
			device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 3); //vykresleni VB jako TriangleStrip
		}

		// hrac prisel na kosticku
		public override bool PlayerOver()
		{
			fadeOut = true;
			doCrossfade = true;
			elapsedTime = 0;

			return false;
		}

		// nastaveni objektu sveta
		public override void setSvet(Svet svet)
		{
			base.setSvet(svet);

			// nalezneme vsechny teleporty
			for (int x = 0; x < svet.width; x++)
			{
				for (int y = 0; y < svet.height; y++)
				{
					if (svet.map[y][x] == '+')
					{
						teleports.Add(new intXY(x, y));
					}
				}
			}
		}

		// funkce se stara o skok
		public void PlayerJump()
		{
			// nalezneme nasledujici teleport v mape
			int index = -1;
			for (int i = 0; i < teleports.Count; i++)
			{
				if (((intXY) teleports[i]).x == objectX && ((intXY) teleports[i]).y == objectY)
				{
					index = i;
					break;
				}
			}

			index++;

			if (index >= teleports.Count)
				index = 0;

			// a nastavime pozici
			svet.setHrac(-(((intXY) teleports[index]).y * 2 + 1), -(((intXY) teleports[index]).x * 2 + 1), true);
		}
	}
}